@@ -1,28 +1,31 @@
 import datetime
-from flask import request, current_app, _request_ctx_stack
-from flask_restful import reqparse, Resource, abort, marshal
-from flask_restful.fields import Boolean, String, Integer, DateTime, Raw
+
+from flask import request, current_app
+from flask_restful import reqparse, Resource as RestfulResource, abort, marshal
 from flask_sqlalchemy import BaseQuery, Pagination, get_state
 from flask.views import MethodViewType
+import itertools
+import sqlalchemy.types as sa_types
 from sqlalchemy.dialects import postgres
 from sqlalchemy.orm import class_mapper
 from sqlalchemy.orm.exc import NoResultFound
-from flask_presst.signals import before_create_item, after_create_item, before_create_relationship, \
-    after_create_relationship, before_delete_relationship, after_delete_relationship, before_update_item, \
-    before_delete_item, after_delete_item, after_update_item, on_filter_read, on_filter_update, \
-    on_filter_delete
-from flask_presst.fields import _RelationshipField, Array, KeyValue, Date, JSON
-from flask_presst.nesting import NestedProxy
-from flask_presst.parsing import PresstArgument
+from sqlalchemy.util import classproperty, OrderedDict
 import six
 
+from flask_presst.routes import ResourceRoute, route
+from flask_presst.fields import String, Integer, Boolean, List, DateTime, EmbeddedBase, Raw, KeyValue, Arbitrary, \
+    Date, Number
+from flask_presst.references import EmbeddedJob, ItemListWrapper, ItemWrapper
+from flask_presst.signals import *
+from flask_presst.parse import SchemaParser
+
 
 LINK_HEADER_FORMAT_STR = '<{0}?page={1}&per_page={2}>; rel="{3}"'
 
 
-class PresstResourceMeta(MethodViewType):
+class ResourceMeta(MethodViewType):
     def __new__(mcs, name, bases, members):
-        class_ = super(PresstResourceMeta, mcs).__new__(mcs, name, bases, members)
+        class_ = super(ResourceMeta, mcs).__new__(mcs, name, bases, members)
 
         if hasattr(class_, '_meta'):
             try:
@@ -31,7 +34,7 @@ class PresstResourceMeta(MethodViewType):
                 meta = {}
 
             class_.resource_name = meta.get('resource_name', class_.__name__).lower()
-            class_.nested_types = nested_types = {}
+            class_.routes = routes = dict(getattr(class_, 'routes', {}))
             class_._id_field = meta.get('id_field', 'id')
             class_._required_fields = meta.get('required_fields', [])
             class_._fields = fields = dict()
@@ -39,33 +42,35 @@ class PresstResourceMeta(MethodViewType):
             class_._meta = meta
 
             for name, m in six.iteritems(members):
-                if isinstance(m, (_RelationshipField, NestedProxy)):
-                    m.bound_resource = class_
-                    if m.relationship_name is None:
-                        m.relationship_name = name
+                if isinstance(m, (EmbeddedBase, ResourceRoute)):
+                    m.binding = class_
 
-                if isinstance(m, NestedProxy):
-                    nested_types[m.relationship_name] = m
+                    if m.attribute is None:
+                        m.attribute = name
+
+                if isinstance(m, ResourceRoute):
+                    routes[m.attribute] = m
                 elif isinstance(m, Raw):
-                    fields[m.attribute or name] = m
+                    field_name = m.attribute or name
+                    fields[field_name] = m
 
         return class_
 
 
-class PresstResource(six.with_metaclass(PresstResourceMeta, Resource)):
+class Resource(six.with_metaclass(ResourceMeta, RestfulResource)):
     """
 
     Resource item property fields are defined as as class attributes. e.g.:
 
     .. code-block:: python
 
-        class PersonResource(PresstResource):
+        class PersonResource(Resource):
             name = fields.String()
             age = fields.Integer()
             # ...
 
-    Each new subclass of :class:`PresstResource` can be configured using a class attribute, :class:`Meta`, which
-    includes properties that are applied at creation time by the :class:`PresstResource`'s metaclass. The following
+    Each new subclass of :class:`Resource` can be configured using a class attribute, :class:`Meta`, which
+    includes properties that are applied at creation time by the :class:`Resource`'s metaclass. The following
     attributes can be declared within :class:`Meta`:
 
     =====================  ==============================================================================
@@ -74,59 +79,167 @@ class PresstResource(six.with_metaclass(PresstResourceMeta, Resource)):
     resource_name          The name of the resource used to build the resource endpoint, also used
                            for referencing the resource using e.g. :class:`fields.ToMany`. *Default:
                            the lower-case of the class name of the resource*
-    id_field               The default implementation of :class:`PresstResource` attempts to read the id
+    id_field               The default implementation of :class:`Resource` attempts to read the id
                            of each resource item using this attribute or item key. The id field will
                            never be marshalled [#f1]_. *Default: 'id'*
     required_fields        A list of fields that must be given in `POST` requests.
     read_only_fields       A list of fields that are returned by the resource but are ignored in `POST`
                            and `PATCH` requests. Useful for e.g. timestamps.
+    title                  JSON-schema title declaration
+    description            JSON-schema description declaration
     =====================  ==============================================================================
 
     .. rubric:: Footnotes
 
     .. [#f1] Adventurous people can override :meth:`marshal_item` to include `id_field` instead of or in addition to
-       ``'resource_uri'``.
+       ``'_uri'``.
 
     """
     api = None
     resource_name = None
-    nested_types = None
+    routes = {}
+    route_prefix = None
 
     _meta = None
     _id_field = None
     _fields = None
+    _relationships = None
     _read_only_fields = None
     _required_fields = None
 
+    #schema = ResourceSchema()
+
+    @route('GET')
+    def schema(self):
+        # TODO enforce Content-Type: application/schema+json (overwritten by Flask-RESTful)
+
+        schema = OrderedDict()
+
+        for schema_property in ('title', 'description'):
+            if schema_property in self._meta:
+                schema[schema_property] = self._meta[schema_property]
+
+        links = [
+            {
+                'rel': 'self',
+                'href': self.api._complete_url('{}/{{id}}'.format(self.route_prefix), ''),
+                'method': 'GET',
+            },
+            {
+                'rel': 'instances',
+                'href': self.api._complete_url('{}'.format(self.route_prefix), ''),
+                'method': 'GET',
+                'schema': {
+                    '$ref': self.api._complete_url('/schema#/definitions/_pagination', '')
+                }
+            }
+        ]
+
+        links = itertools.chain(links, *[route.get_links(self) for name, route in sorted(self.routes.items())
+                                         if name != 'schema'])
+
+        schema['type'] = 'object'
+        schema['definitions'] = definitions = {}
+        schema['properties'] = properties = {}
+        schema['required'] = self._required_fields
+        schema['links'] = list(links)
+
+        # fields:
+        for name, field in sorted(self._fields.items()):
+            definition = field.schema
+
+            if '$ref' in definition:
+                properties[name] = definition
+                continue
+
+            if name in self._read_only_fields:
+                definition['readOnly'] = True
+
+            definitions[name] = definition
+            properties[name] = {'$ref': '#/definitions/{}'.format(name)}
+
+        definitions['_uri'] = {
+            'type': 'string',
+            'format': 'uri',
+            'readOnly': True
+        }
+
+        properties['_uri'] = {
+            '$ref': '#/definitions/_uri'
+        }
+
+        return schema
+
     def get(self, id=None, **kwargs):
         if id is None:
-            item_list = self.get_item_list()
-            return self.marshal_item_list(item_list)
+            return ItemListWrapper.get_list(self).marshal()
         else:
-            item = self.get_item_for_id(id)
-            return self.marshal_item(item)
+            return ItemWrapper.read(self, id).marshal()
 
     def post(self, id=None, *args, **kwargs):
         if id is None:
-            return self.marshal_item(self.create_item(self.request_parse_item()))
+            return ItemListWrapper.create(self, request.json).marshal(), 200
         else:
-            return self.marshal_item(self.update_item(id, self.request_parse_item()))
+            return ItemWrapper.read(self, id).update(request.json).marshal(), 200
 
-    def patch(self, id):
+    def patch(self, id=None):
         if id is None:
-            abort(400, message='PATCH is not permitted on collections.')
+            abort(405, message='PATCH is not permitted on collections')
         else:
-            # TODO consider abort(400) if request.JSON is not a dictionary.
-            changes = self.request_parse_item(limit_fields=(name for name in self._fields if name in request.json))
-            return self.marshal_item(self.update_item(id, changes, partial=True))
+            return ItemWrapper.read(self, id).update(request.json, partial=True).marshal(), 200
 
-    def delete(self, id, *args, **kwargs):
+    def delete(self, id=None, *args, **kwargs):
         if id is None:
-            abort(400, message='DELETE is not permitted on collections.')
+            abort(405, message='DELETE is not permitted on collections.')
         else:
-            self.delete_item(id)
+            ItemWrapper.read(self, id).delete()
             return None, 204
 
+    @classmethod
+    def get_item_from_uri(cls, value, changes=None):
+        resource, id = cls.api.parse_resource_uri(value)
+
+        if cls != resource:
+            abort(400, msg='Wrong resource item type, expected {0}, got {1}'.format(
+                cls.resource_name,
+                resource.resource_name))
+
+        return cls.get_item_for_id(id)
+
+    @classproperty
+    def item_parser(cls):
+        return SchemaParser(cls._fields, cls._required_fields, cls._read_only_fields)
+
+    @classmethod
+    def begin(cls):
+        """
+        Called at the beginning of a create or update operation.
+        May be a no-op.
+        """
+        pass
+
+    @classmethod
+    def commit(cls):
+        """
+        Called at the end of a create or update operation.
+        Should flush all changes and fail if necessary.
+        """
+        pass
+
+    def _request_get_data(self):
+        # TODO upcoming in Flask 0.11: 'is_json':
+        # if not request.is_json:
+        #     abort(415)
+        request_data = request.json
+
+        if request_data is None:
+            abort(400, message='JSON required')
+
+        if not isinstance(request_data, (dict, list)):
+            abort(400, message='JSON dictionary or array required')
+
+        return request_data
+
     @classmethod
     def get_item_for_id(cls, id_):  # pragma: no cover
         """
@@ -160,39 +273,39 @@ class PresstResource(six.with_metaclass(PresstResourceMeta, Resource)):
         raise NotImplementedError()
 
     @classmethod
-    def get_item_list_for_relationship(cls, relationship, parent_item):  # pragma: no cover
+    def get_relationship(cls, item, relationship):  # pragma: no cover
         """
-        Return the list of items for a relationship. Must be implemented in nested resources.
+        Return the list of items for a relationship. Must be implemented in the parents of nested resources.
 
-        :param str relationship: name of the relationship in the parent level resource
-        :param parent_item: instance of the item from the parent level resource
+        :param item: instance of the item from the parent level resource
+        :param str relationship: name of the relationship from the parent resource
         """
         raise NotImplementedError()
 
     @classmethod
-    def create_item_relationship(cls, id_, relationship, parent_item):  # pragma: no cover
+    def add_to_relationship(cls, item, relationship, child):  # pragma: no cover
         """
-        Add an item to a relationship. Must be implemented in nested resources.
+        Add a child item to a relationship. Must be implemented in the parents of nested resources.
 
-        :param id_: the id of the item to add to the relationship
-        :param str relationship: name of the relationship in the parent level resource
-        :param parent_item: instance of the item from the parent level resource
+        :param item: instance of the item from the parent resource
+        :param str relationship: name of the relationship from the parent resource
+        :param child: item to remove from the relationship
         """
         raise NotImplementedError()
 
     @classmethod
-    def delete_item_relationship(cls, id_, relationship, parent_item):  # pragma: no cover
+    def remove_from_relationship(cls, item, relationship, child):  # pragma: no cover
         """
-        Delete an item from a relationship. Must be implemented in nested resources.
+        Delete a child item from a relationship. Must be implemented in the parents of nested resources.
 
-        :param id_: the id of the item to remove from the relationship
-        :param str relationship: name of the relationship in the parent level resource
-        :param parent_item: instance of the item from the parent level resource
+        :param item: instance of the item from the parent resource
+        :param str relationship: name of the relationship from the parent resource
+        :param child: item to remove from the relationship
         """
         raise NotImplementedError()
 
     @classmethod
-    def create_item(cls, dct):  # pragma: no cover
+    def create_item(cls, dct, commit=True):  # pragma: no cover
         """
         Must be implemented to create a new item in the resource collection.
 
@@ -202,28 +315,28 @@ class PresstResource(six.with_metaclass(PresstResourceMeta, Resource)):
         raise NotImplementedError()
 
     @classmethod
-    def update_item(cls, id_, dct, partial=False):  # pragma: no cover
+    def update_item(cls, item, changes, partial=False, commit=True):  # pragma: no cover
         """
         Must be implemented to update an item in the resource collection.
 
-        :param id_: id of the item to update
-        :param dict dct: dictionary of changes
+        :param item: item to update
+        :param dict changes: dictionary of changes
         :param bool partial: whether this is a `PATCH` change
         """
         raise NotImplementedError()
 
     @classmethod
-    def delete_item(cls, id_):  # pragma: no cover
+    def delete_item(cls, item):  # pragma: no cover
         """
         Must be implemented to delete an item from the resource collection.
 
-        :param id_: id of the item to delete
+        :param item: item to delete
         """
         raise NotImplementedError()
 
     @classmethod
-    def item_get_resource_uri(cls, item):
-        """Returns the `resource_uri` of an item.
+    def item_get_uri(cls, item):
+        """Returns the `_uri` of an item.
 
         .. seealso:: :meth:`item_get_id()`
         """
@@ -236,7 +349,7 @@ class PresstResource(six.with_metaclass(PresstResourceMeta, Resource)):
         """
         Marshals the item using the resource fields and returns a JSON-compatible dictionary.
         """
-        marshaled = {'resource_uri': cls.item_get_resource_uri(item)}
+        marshaled = {'_uri': cls.item_get_uri(item)}
         marshaled.update(marshal(item, cls._fields))
         return marshaled
 
@@ -249,23 +362,8 @@ class PresstResource(six.with_metaclass(PresstResourceMeta, Resource)):
         """
         return list(cls.marshal_item(item) for item in items)
 
-    def request_parse_item(self, limit_fields=None):
-        """
-        Helper method to parse an item from the request.
 
-        :param limit_fields: optional list of field names to parse; if not set, all fields will be parsed.
-        """
-        parser = reqparse.RequestParser(argument_class=PresstArgument)
-
-        for name in limit_fields or self._fields: # FIXME handle this in PresstArgument.
-            if name not in self._read_only_fields:
-                required = name in self._required_fields
-                parser.add_argument(name, type=self._fields[name], required=required, ignore=not required)
-
-        return parser.parse_args()
-
-
-class ModelResourceMeta(PresstResourceMeta):
+class ModelResourceMeta(ResourceMeta):
     def __new__(mcs, name, bases, members):
         class_ = super(ModelResourceMeta, mcs).__new__(mcs, name, bases, members)
         meta = class_._meta
@@ -302,45 +400,44 @@ class ModelResourceMeta(PresstResourceMeta):
                         (exclude_fields and name not in exclude_fields) or \
                         not (include_fields or exclude_fields):
 
-                    field_class = None
-
-                    if meta.get('exclude_polymorphic', False) and column.table != mapper.tables[-1]:
-                        continue
-
                     if column.primary_key or column.foreign_keys:
                         continue
 
-                    if isinstance(column.type, postgres.ARRAY):
-                        field_type = list
-                    elif isinstance(column.type, postgres.HSTORE):
-                        field_type = dict
-                        field_class = KeyValue
-                    elif hasattr(postgres, 'JSON') and isinstance(column.type, postgres.JSON):
-                        field_type = lambda data: data
-                        field_class = JSON
-                    else:
-                        field_type = column.type.python_type
+                    field_args = ()
+                    field_kwargs = {}
 
-                    field_types[name] = field_type
+                    if isinstance(column.type, postgres.ARRAY):
+                        field_class = List
+                        field_args = (String,)
+                    elif isinstance(column.type, sa_types.String) and column.type.length:
+                        field_class = String
+                        field_kwargs = {'max_length': column.type.length}
+                    elif isinstance(column.type, postgres.HSTORE):
+                        field_class = KeyValue
+                        field_args = (String,)
+                    elif hasattr(postgres, 'JSON') and isinstance(column.type, postgres.JSON):
+                        field_class = Arbitrary
+                    else:
+                        field_class = class_._get_field_from_python_type(column.type.python_type)
 
                     # Add to list of fields.
                     if not name in fields:
-                        if not field_class:
-                            field_class = class_._get_field_from_python_type(field_type)
+                        field_kwargs['nullable'] = column.nullable
 
-                        # TODO implement support for ColumnDefault
-                        # fields[name] = field_class(default=column.default)
-                        fields[name] = field_class()
+                        if column.default is not None and column.default.is_scalar:
+                            field_kwargs['default'] = column.default.arg
+
+                        fields[name] = field_class(*field_args, **field_kwargs)
 
                         if not (column.nullable or column.default):
                             required_fields.append(name)
         return class_
 
 
-class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
+class ModelResource(six.with_metaclass(ModelResourceMeta, Resource)):
     """
 
-    :class:`ModelResource` inherits all of the :class:`Meta` options of :class:`PresstResource`, however
+    :class:`ModelResource` inherits all of the :class:`Meta` options of :class:`Resource`, however
     with slightly different behavior and including some additions.
 
     =====================  ==============================================================================
@@ -364,11 +461,10 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
     =====================  ==============================================================================
 
 
-    This resource class processes all of the signals in :mod:`flask.ext.presst.signals`.
+    This resource class processes all of the signals in :mod:`flask_presst.signals`.
     """
     _model = None
     _model_id_column = None
-    _field_types = None
 
     @staticmethod
     def _get_field_from_python_type(python_type):
@@ -376,8 +472,9 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
             str: String,
             six.text_type: String,
             int: Integer,
+            float: Number,
             bool: Boolean,
-            list: Array,
+            list: List,
             dict: KeyValue,
             datetime.date: Date,
             datetime.datetime: DateTime
@@ -392,21 +489,17 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
         return cls._model
 
     @classmethod
-    def _process_filter_signal(cls, query, **kwargs):
-        if request.method in ('HEAD', 'GET'):
-            signal = on_filter_read
-        elif request.method in ('POST', 'PATCH'):
-            signal = on_filter_update
-        elif request.method in ('DELETE',):
-            signal = on_filter_delete
-        else:
-            return query
+    def begin(cls):
+        cls._get_session()
 
-        for _, response in signal.send(cls, **kwargs):
-            if callable(response):
-                query = response(query)
+    @classmethod
+    def commit(cls):
+        # TODO handle errors
+        cls._get_session().commit()
 
-        return query
+    @classmethod
+    def rollback(cls):
+        cls._get_session().rollback()
 
     @classmethod
     def get_item_list(cls):
@@ -419,63 +512,50 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
         if isinstance(query, list):
             abort(500, message='Nesting not supported for this resource.')
 
-        return cls._process_filter_signal(query)
+        return query
 
     @classmethod
-    def get_item_list_for_relationship(cls, relationship, parent_item):
-        query = getattr(parent_item, relationship)
+    def get_relationship(cls, item, relationship):
+        query = getattr(item, relationship)
 
         if isinstance(query, list):
             abort(500, message='Nesting not supported for this resource.')
 
-        return cls._process_filter_signal(query)
+        # FIXME build dynamic query with backref
+
+        return query
 
     @classmethod
-    def create_item_relationship(cls, id_, relationship, parent_item):
-        item = cls.get_item_for_id(id_)
+    def add_to_relationship(cls, item, relationship, child):
 
-        before_create_relationship.send(cls,
-                                        parent_item=parent_item,
-                                        relationship=relationship,
-                                        item=item)
+        before_add_relationship.send(cls,
+                                     item=item,
+                                     relationship=relationship,
+                                     child=child)
 
-        session = cls._get_session()
+        getattr(item, relationship).append(child)
 
-        try:
-            getattr(parent_item, relationship).append(item)
-            session.commit()
-        except:
-            session.rollback()
-            raise
+        after_add_relationship.send(cls,
+                                    item=item,
+                                    relationship=relationship,
+                                    child=child)
 
-        after_create_relationship.send(cls,
-                                       parent_item=parent_item,
-                                       relationship=relationship,
-                                       item=item)
-        return item
+        return child
 
     @classmethod
-    def delete_item_relationship(cls, id_, relationship, parent_item):
-        item = cls.get_item_for_id(id_)
+    def remove_from_relationship(cls, item, relationship, child):
 
-        before_delete_relationship.send(cls,
-                                        parent_item=parent_item,
+        before_remove_relationship.send(cls,
+                                        item=item,
                                         relationship=relationship,
-                                        item=item)
+                                        child=child)
 
-        session = cls._get_session()
+        getattr(item, relationship).remove(child)
 
-        try:
-            getattr(parent_item, relationship).remove(item)
-            session.commit()
-        except:
-            session.rollback()
-            raise
-
-        after_delete_relationship.send(cls,
-                                       parent_item=parent_item,
+        after_remove_relationship.send(cls,
+                                       item=item,
                                        relationship=relationship,
-                                       item=item)
+                                       child=child)
 
     @classmethod
     def get_item_for_id(cls, id_):
@@ -485,10 +565,11 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
             abort(404)
 
     @classmethod
-    def create_item(cls, dct):
+    def create_item(cls, properties, commit=True):
         # noinspection PyCallingNonCallable
         item = cls._model()
-        for key, value in six.iteritems(dct):
+
+        for key, value in six.iteritems(properties):
             setattr(item, key, value)
 
         before_create_item.send(cls, item=item)
@@ -497,7 +578,8 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
 
         try:
             session.add(item)
-            session.commit()
+            if commit:
+                session.commit()
         except:
             session.rollback()
             raise
@@ -506,18 +588,17 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
         return item
 
     @classmethod
-    def update_item(cls, id_, dct, partial=False):
-        item = cls.get_item_for_id(id_)
-
+    def update_item(cls, item, changes, partial=False, commit=True):
         session = cls._get_session()
 
         try:
-            before_update_item.send(cls, item=item, changes=dct, partial=partial)
+            before_update_item.send(cls, item=item, changes=changes, partial=partial)
 
-            for key, value in six.iteritems(dct):
+            for key, value in six.iteritems(changes):
                 setattr(item, key, value)
 
-            session.commit()
+            if commit:
+                session.commit()
         except:
             session.rollback()
             raise
@@ -526,9 +607,7 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
         return item
 
     @classmethod
-    def delete_item(cls, id_):
-        item = cls.get_item_for_id(id_)
-
+    def delete_item(cls, item):
         before_delete_item.send(cls, item=item)
 
         session = cls._get_session()
@@ -582,40 +661,3 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
 
         # fallback:
         return super(ModelResource, cls).marshal_item_list(item_list)
-
-
-class PolymorphicModelResource(ModelResource):
-    """
-    :class:`PolymorphicModelResource` is identical to :class:`ModelResource`, except that when it marshals an item
-    that has a different class than the ``model`` attribute defined in :class:`Meta`, it marshals the contents of that
-    model separately from the inherited resource and adds it to the marshalled dictionary as a property with the
-    name of the inherited resource.
-
-    e.g.
-
-    .. code-block:: javascript
-
-        {
-            "resource_uri": "/polymorphic_resource/1",
-            // polymorphic_resource properties
-            "base_resource": {
-                "resource_uri": "/base_resource/1",
-                // base_resource properties
-            }
-        }
-
-
-    :class:`PolymorphicModelResource` is designed to be used with SQLAlchemy models that
-    make use of `SQLAlchemy's polymorphic inheritance <http://docs.sqlalchemy.org/en/latest/orm/inheritance.html>`_.
-    """
-
-    @classmethod
-    def marshal_item(cls, item):
-        resource = cls.api.get_resource_for_model(item.__class__)
-        marshaled = super(PolymorphicModelResource, cls).marshal_item(item)
-
-        if resource and resource != cls:
-            marshaled[resource.resource_name.replace('/', '__')] = resource.marshal_item(item)
-
-        # fallback:
-        return marshaled
\ No newline at end of file
